// 
// UrsTimeServer.cpp 
// 
#include <ESP8266WiFi.h>
#include <UrsSystemTimeServer.h>

// --------------------------------------------------------
// ---  Timezone ------------------------------------------
// --------------------------------------------------------
//Mitteleuropische Zeit MEZ (Frankfurt, Paris, Berlin) = Central European Time (CET)
TimeChangeRule tcrMESZ = { "MESZ", Last, Sun, Mar, 2, 120 };     // Mitteleuropische Sommerzeit
TimeChangeRule tcrMEZ = { "MEZ ", Last, Sun, Oct, 3, 60 };       // Mitteleuropische Zeit
Timezone MEZ(tcrMESZ, tcrMEZ);

// --------------------------------------------------------
// ---  UrsTimeServer -----------------------------------
// --------------------------------------------------------

void UrsSystemTimeServer::setTime(time_t t) {
  sysTime = t;
  nextSyncTime = t + syncInterval;
  State = UrsSystemTimeState::timeSet;
  prevMillis = millis();  // Mit der Zhlung der bereits bercksichtigten Zeit neu beginnen
}

time_t UrsSystemTimeServer::now() {
  // Systemzeit nachfahren
  while (millis() - prevMillis >= 1000) {
    sysTime++;
    prevMillis += 1000;
  }
  return sysTime + debugOffset;
}

UrsTimeElements UrsSystemTimeServer::localCurrentTime() {
  if (State == UrsSystemTimeState::timeNotSet) // Leeres Element zurckliefern
    return UrsTimeElements();

  // Zeiger auf eine TimeChangeRule-Instanz zur Rckmeldung der gltigen Zeitzonen-Bezeichnung (Sommer-, Standardzeit)
  TimeChangeRule *tcr;

  time_t local = timezone.toLocal(now(), &tcr); // UTC in lokale Zeit wandeln
  tmElements_t tme;
  breakTime(local, tme);  // time_t in seine Elemente auflsen

  return UrsTimeElements(tme.Second, tme.Minute, tme.Hour, tme.Wday, tme.Day, tme.Month, tme.Year + 1970, State, tcr->abbrev, tcr->offset * 60);
}

int UrsSystemTimeServer::currentOffsetUTC() {
  auto t = localCurrentTime();
  return t.OffsetUTC;
}

UrsTimeElements UrsSystemTimeServer::toLocalTime(time_t utc) {
  // Zeiger auf eine TimeChangeRule-Instanz zur Rckmeldung der gltigen Zeitzonen-Bezeichnung (Sommer-, Standardzeit)
  TimeChangeRule *tcr;

  time_t local = timezone.toLocal(utc, &tcr); // UTC in lokale Zeit wandeln
  tmElements_t tme;
  breakTime(local, tme);  // time_t in seine Elemente auflsen

  return UrsTimeElements(tme.Second, tme.Minute, tme.Hour, tme.Wday, tme.Day, tme.Month, tme.Year + 1970, State, tcr->abbrev, tcr->offset * 60);
}

bool UrsSystemTimeServer::synchronize() {
  // Systemzeit nachfahren
  while (millis() - prevMillis >= 1000) {
    sysTime++;
    prevMillis += 1000;
  }

  UrsTimeSourceState st = timeSource.loadTime(); // Synchronisation durchfhren
  if (st == UrsTimeSourceState::success) { // Erfolgreich?
    setTime(timeSource.getCurrent()); // Status wird von setTime gesetzt
    nextSyncTime = sysTime + syncInterval;
    return true;
  }
  else {
    nextSyncTime = sysTime + syncErrorInterval;
    State = (State == UrsSystemTimeState::timeNotSet) ? UrsSystemTimeState::timeNotSet : UrsSystemTimeState::timeNeedsSync;
    return false;
  }
}

void UrsSystemTimeServer::setDebugOffsetFromTime(time_t UTC) { 
  now(); // millis nachfahren
  debugOffset = UTC - sysTime;
}


void UrsSystemTimeServer::handle() {
  if (sysTime >= nextSyncTime) { // Zeit ist abgelaufen
    synchronize();
  }
}