// UrsTimeElements.h
//
// s. UrsTime.h

#ifndef _URSTIMEELEMENTS_h
#define _URSTIMEELEMENTS_h

#include <Arduino.h>
#include <Printable.h>
#include <UrsTime.h>
#include <time.h>

enum class UrsTimeLang {standard, en, de};

// Aufgelste Form eines Zeitpunkts
class UrsTimeElements : public Printable {
private:
  static UrsTimeLang defaultLang;
  static String defaultFormat;
  String monthStrDE(const uint8_t month) const;      // Monat als String (z.B. Januar)
  String monthShortStrDE(const uint8_t month) const; // Monatskurzbezeichnung (z.B. Jan)
  String dayStrDE(const uint8_t day) const;          // Tagesname als String (z.B. Sonntag)
  String dayShortStrDE(const uint8_t day) const;     // Tageskurzbezeichnung (z.B. Son)

public:
  UrsTimeElements(const uint8_t s, const uint8_t m, const uint8_t h, const uint8_t w, const uint8_t d,
                  const uint8_t mo, const uint16_t y, const UrsSystemTimeState Status, const String& TZ, const int utcOff) :
    second(s), minute(m), hour(h), wDay(w), day(d), month(mo), year(y), State(Status), TimeZone(TZ), OffsetUTC(utcOff) {
  }
  UrsTimeElements() : State(UrsSystemTimeState::timeNotSet) {}

  uint8_t second;
  uint8_t minute;
  uint8_t hour;
  uint8_t wDay;   // Wochentag, Sonntag hat den Wert 1
  uint8_t day;
  uint8_t month;
  uint16_t year;
  UrsSystemTimeState State; // Synchronisationszustand des Zeitservers bei der Abfrage der Zeit.
                            // Die anderen Werte sind nur dann gltig, State != timeNotSet ist.
  String TimeZone;          // Bezeichnung der aktuellen Zeitzone (z.B. MEZ)
  int OffsetUTC;            // Offset der Timezone zu UTC in Sekunden


  void setDefaultLanguage(UrsTimeLang dl); // Legt die Standard-Sprache fest ('standard' ndert nichts!). Default = de.
  UrsTimeLang getDefaultLanguage() { return defaultLang; }; // Ruft die Standardsprache ab.
  void setDefaultFormat(String df);        // Legt das Standardformat fr Datumsangaben fest. Default = "%Y-%m-%d %H:%M:%S"
  String getDefaultFormat() { return defaultFormat; }; // Ruf das Standardformat fr Datumsangaben ab.


  String monthStr(const UrsTimeLang l = UrsTimeLang::standard) const;      // Monat als String (z.B. Januar)
  String monthShortStr(const UrsTimeLang l = UrsTimeLang::standard) const; // Monatskurzbezeichnung (z.B. Jan)
  String dayStr(const UrsTimeLang l = UrsTimeLang::standard) const;        // Tagesname als String (z.B. Sonntag)
  String dayShortStr(const UrsTimeLang l = UrsTimeLang::standard) const;   // Tageskurzbezeichnung (z.B. So)

  // Die nachfolgenden Methoden formattieren zu einem String gem der Format-Angabe. Folgende Symbole sind mglich:
  /*
  %a	Abgekrzter Namer des Wochentags
  %A	Voller Wochentagsname
  %b	Abgekrzter Name des Monats
  %B	Vollstndiger Monatsname
  %d	Tag des Monats als Dezimalzahl(01 - 31)
  %H	Die Stunde im 24 - Stunden - Format(00  23)
  %I	Stundenangabe im 12 - Stunden - Format(01 - 12)
  %m	Monatsangabe als Dezimalzahl(01 - 12)
  %M	Minutenangabe als Dezimalzahl(00  59)
  %p	A.M. / P.M. - Kennung des aktuellen Gebietsschemas.fr 12 - Stunden - Uhr
  %S	Sekundenangabe als Dezimalzahl(00  59)
  %w	Angabe des Wochentags als Dezimalzahl(0  6; Sonntag ist 0)
  %y	Jahresangabe ohne Jahrhundert als Dezimalzahl(00 - 99)
  %Y	Jahresangabe mit Jahrhundert als Dezimalzahl
  %z, %Z	Zeitzonenname oder Abkrzung der Zeitzone, je nach Registrierungseintrge; keine Zeichen, wenn Zeitzone unbekannt ist
  %%	Prozentzeichen
*/
  String toString(const String format, const UrsTimeLang l = UrsTimeLang::standard) const;
  String toString() const;
  virtual size_t printTo(Print& p) const;
};

#endif

